=== modified file 'NEWS'
--- NEWS	2007-12-19 06:04:19 +0000
+++ NEWS	2007-12-19 08:06:07 +0000
@@ -85,6 +85,39 @@
 
   TESTING:
 
+   * Modules can now customise their tests by defining a ``load_tests``
+     attribute. ``pydoc bzrlib.tests.TestUtil.TestLoader.loadTestsFromModule``
+     for the documentation on this attribute. (Robert Collins)
+
+   * New helper function ``bzrlib.tests.condition_id_re`` which helps
+     filter tests based on a regular expression search on the tests id.
+     (Robert Collins)
+    
+   * New helper function ``bzrlib.tests.condition_isinstance`` which helps
+     filter tests based on class. (Robert Collins)
+    
+   * New helper function ``bzrlib.tests.exclude_suite_by_condition`` which
+     generalises the ``exclude_suite_by_re`` function. (Robert Collins)
+
+   * New helper function ``bzrlib.tests.filter_suite_by_condition`` which
+     generalises the ``filter_suite_by_re`` function. (Robert Collins)
+
+   * New helper method ``bzrlib.tests.exclude_tests_by_re`` which gives a new
+     TestSuite that does not contain tests from the input that matched a
+     regular expression. (Robert Collins)
+
+   * New helper method ``bzrlib.tests.randomize_suite`` which returns a
+     randomized copy of the input suite. (Robert Collins)
+
+   * New helper method ``bzrlib.tests.split_suite_by_re`` which splits a test
+     suite into two according to a regular expression. (Robert Collins)
+
+   * The ``exclude_pattern`` and ``random_order`` parameters to the function
+     ``bzrlib.tests.filter_suite_by_re`` have been deprecated. (Robert Collins)
+
+   * The method ``bzrlib.tests.sort_suite_by_re`` has been deprecated. It is 
+     replaced by the new helper methods added in this release. (Robert Collins)
+
 
 bzr 1.0 2007-12-14
 ==================

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2007-11-18 19:56:39 +0000
+++ bzrlib/dirstate.py	2007-12-19 08:12:34 +0000
@@ -250,7 +250,7 @@
 
     A dirstate is a specialised data structure for managing local working
     tree state information. Its not yet well defined whether it is platform
-    specific, and if it is how we detect/parameterise that.
+    specific, and if it is how we detect/parameterize that.
 
     Dirstates use the usual lock_write, lock_read and unlock mechanisms.
     Unlike most bzr disk formats, DirStates must be locked for reading, using

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2007-12-07 06:36:56 +0000
+++ bzrlib/remote.py	2007-12-19 08:12:34 +0000
@@ -197,7 +197,7 @@
     Instances of this repository are represented by RemoteRepository
     instances.
 
-    The RemoteRepositoryFormat is parameterised during construction
+    The RemoteRepositoryFormat is parameterized during construction
     to reflect the capabilities of the real, remote format. Specifically
     the attributes rich_root_data and supports_tree_reference are set
     on a per instance basis, and are not set (and should not be) at

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2007-12-18 19:42:10 +0000
+++ bzrlib/repofmt/pack_repo.py	2007-12-19 08:12:34 +0000
@@ -2129,7 +2129,7 @@
 
 
 class RepositoryFormatKnitPack1(RepositoryFormatPack):
-    """A no-subtrees parameterised Pack repository.
+    """A no-subtrees parameterized Pack repository.
 
     This format was introduced in 0.92.
     """
@@ -2159,7 +2159,7 @@
 
 
 class RepositoryFormatKnitPack3(RepositoryFormatPack):
-    """A subtrees parameterised Pack repository.
+    """A subtrees parameterized Pack repository.
 
     This repository format uses the xml7 serializer to get:
      - support for recording full info about the tree root
@@ -2201,7 +2201,7 @@
 
 
 class RepositoryFormatKnitPack4(RepositoryFormatPack):
-    """A rich-root, no subtrees parameterised Pack repository.
+    """A rich-root, no subtrees parameterized Pack repository.
 
     This repository format uses the xml6 serializer to get:
      - support for recording full info about the tree root

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-12-18 22:21:26 +0000
+++ bzrlib/repository.py	2007-12-19 08:12:34 +0000
@@ -1964,7 +1964,7 @@
     _matchingbzrdir - the bzrdir format that the repository format was
     originally written to work with. This can be used if manually
     constructing a bzrdir and repository, or more commonly for test suite
-    parameterisation.
+    parameterization.
     """
 
     # Set to True or False in derived classes. True indicates that the format

=== modified file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	2007-04-23 03:41:48 +0000
+++ bzrlib/tests/TestUtil.py	2007-12-18 07:29:14 +0000
@@ -74,7 +74,8 @@
 
 
 class TestLoader(unittest.TestLoader):
-    """Custom  TestLoader to address some quirks in the stock python one."""
+    """Custom TestLoader to extend the stock python one."""
+
     suiteClass = TestSuite
 
     def loadTestsFromModuleNames(self, names):
@@ -86,10 +87,41 @@
         """
         result = self.suiteClass()
         for name in names:
-            _load_module_by_name(name)
-            result.addTests(self.loadTestsFromName(name))
+            module = _load_module_by_name(name)
+            result.addTests(self.loadTestsFromModule(module))
         return result
 
+    def loadTestsFromModule(self, module):
+        """Load tests from a module object.
+
+        This extension of the python test loader looks for an attribute
+        load_tests in the module object, and if not found falls back to the
+        regular python loadTestsFromModule.
+
+        If a load_tests attribute is found, it is called and the result is
+        returned. 
+
+        load_tests should be defined like so:
+        >>> def load_tests(standard_tests, module, loader):
+        >>>    pass
+
+        standard_tests is the tests found by the stock TestLoader in the
+        module, module and loader are the module and loader instances.
+
+        For instance, to run every test twice, you might do:
+        >>> def load_tests(standard_tests, module, loader):
+        >>>     result = loader.suiteClass()
+        >>>     for test in iter_suite_tests(standard_tests):
+        >>>         result.addTests([test, test])
+        >>>     return result
+        """
+        basic_tests = super(TestLoader, self).loadTestsFromModule(module)
+        load_tests = getattr(module, "load_tests", None)
+        if load_tests is not None:
+            return load_tests(basic_tests, module, self)
+        else:
+            return basic_tests
+
 
 def _load_module_by_name(mod_name):
     parts = mod_name.split('.')

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-12-17 17:28:25 +0000
+++ bzrlib/tests/__init__.py	2007-12-19 08:12:34 +0000
@@ -78,9 +78,13 @@
 import bzrlib.store
 from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_function,
     deprecated_method,
+    deprecated_passed,
     zero_ninetyone,
     zero_ninetytwo,
+    one_zero,
     )
 import bzrlib.trace
 from bzrlib.transport import get_transport
@@ -95,7 +99,6 @@
                           TestSuite,
                           TestLoader,
                           )
-from bzrlib.tests.EncodingAdapter import EncodingTestAdapter
 from bzrlib.tests.treeshape import build_tree_contents
 import bzrlib.version_info_formats.format_custom
 from bzrlib.workingtree import WorkingTree, WorkingTreeFormat2
@@ -1717,8 +1720,8 @@
     _TEST_NAME = 'test'
 
     def __init__(self, methodName='runTest'):
-        # allow test parameterisation after test construction and before test
-        # execution. Variables that the parameteriser sets need to be 
+        # allow test parameterization after test construction and before test
+        # execution. Variables that the parameterizer sets need to be 
         # ones that are not set by setUp, or setUp will trash them.
         super(TestCaseWithMemoryTransport, self).__init__(methodName)
         self.vfs_transport_factory = default_transport
@@ -2234,51 +2237,185 @@
             self.transport_readonly_server = HttpServer
 
 
-def filter_suite_by_re(suite, pattern, exclude_pattern=None,
-                       random_order=False):
+def condition_id_re(pattern):
+    """Create a condition filter which performs a re check on a test's id.
+    
+    :param pattern: A regular expression string.
+    :return: A callable that returns True if the re matches.
+    """
+    filter_re = re.compile(pattern)
+    def condition(test):
+        test_id = test.id()
+        return filter_re.search(test_id)
+    return condition
+
+
+def condition_isinstance(klass_or_klass_list):
+    """Create a condition filter which returns isinstance(param, klass).
+    
+    :return: A callable which when called with one parameter obj return the
+        result of isinstance(obj, klass_or_klass_list).
+    """
+    def condition(obj):
+        return isinstance(obj, klass_or_klass_list)
+    return condition
+
+
+def exclude_tests_by_condition(suite, condition):
+    """Create a test suite which excludes some tests from suite.
+
+    :param suite: The suite to get tests from.
+    :param condition: A callable whose result evaluates True when called with a
+        test case which should be excluded from the result.
+    :return: A suite which contains the tests found in suite that fail
+        condition.
+    """
+    result = []
+    for test in iter_suite_tests(suite):
+        if not condition(test):
+            result.append(test)
+    return TestUtil.TestSuite(result)
+
+
+def filter_suite_by_condition(suite, condition):
+    """Create a test suite by filtering another one.
+    
+    :param suite: The source suite.
+    :param condition: A callable whose result evaluates True when called with a
+        test case which should be included in the result.
+    :return: A suite which contains the tests found in suite that pass
+        condition.
+    """ 
+    result = []
+    for test in iter_suite_tests(suite):
+        if condition(test):
+            result.append(test)
+    return TestUtil.TestSuite(result)
+
+
+def filter_suite_by_re(suite, pattern, exclude_pattern=DEPRECATED_PARAMETER,
+                       random_order=DEPRECATED_PARAMETER):
     """Create a test suite by filtering another one.
     
     :param suite:           the source suite
     :param pattern:         pattern that names must match
-    :param exclude_pattern: pattern that names must not match, if any
-    :param random_order:    if True, tests in the new suite will be put in
-                            random order
+    :param exclude_pattern: A pattern that names must not match. This parameter
+        is deprecated as of bzrlib 1.0. Please use the separate function
+        exclude_tests_by_re instead.
+    :param random_order:    If True, tests in the new suite will be put in
+        random order. This parameter is deprecated as of bzrlib 1.0. Please
+        use the separate function randomize_suite instead.
     :returns: the newly created suite
     """ 
-    return sort_suite_by_re(suite, pattern, exclude_pattern,
-        random_order, False)
-
-
+    if deprecated_passed(exclude_pattern):
+        symbol_versioning.warn(
+            one_zero % "passing exclude_pattern to filter_suite_by_re",
+                DeprecationWarning, stacklevel=2)
+        if exclude_pattern is not None:
+            suite = exclude_tests_by_re(suite, exclude_pattern)
+    condition = condition_id_re(pattern)
+    result_suite = filter_suite_by_condition(suite, condition)
+    if deprecated_passed(random_order):
+        symbol_versioning.warn(
+            one_zero % "passing random_order to filter_suite_by_re",
+                DeprecationWarning, stacklevel=2)
+        if random_order:
+            result_suite = randomize_suite(result_suite)
+    return result_suite
+
+
+def exclude_tests_by_re(suite, pattern):
+    """Create a test suite which excludes some tests from suite.
+
+    :param suite: The suite to get tests from.
+    :param pattern: A regular expression string. Test ids that match this
+        pattern will be excluded from the result.
+    :return: A TestSuite that contains all the tests from suite without the
+        tests that matched pattern. The order of tests is the same as it was in
+        suite.
+    """
+    return exclude_tests_by_condition(suite, condition_id_re(pattern))
+
+
+def preserve_input(something):
+    """A helper for performing test suite transformation chains.
+
+    :param something: Anything you want to preserve.
+    :return: Something.
+    """
+    return something
+
+
+def randomize_suite(suite):
+    """Return a new TestSuite with suite's tests in random order.
+    
+    The tests in the input suite are flattened into a single suite in order to
+    accomplish this. Any nested TestSuites are removed to provide global
+    randomness.
+    """
+    tests = list(iter_suite_tests(suite))
+    random.shuffle(tests)
+    return TestUtil.TestSuite(tests)
+
+
+@deprecated_function(one_zero)
 def sort_suite_by_re(suite, pattern, exclude_pattern=None,
                      random_order=False, append_rest=True):
-    """Create a test suite by sorting another one.
+    """DEPRECATED: Create a test suite by sorting another one.
+
+    This method has been decomposed into separate helper methods that should be
+    called directly:
+     - filter_suite_by_re
+     - exclude_tests_by_re
+     - randomize_suite
+     - split_suite_by_re
     
     :param suite:           the source suite
     :param pattern:         pattern that names must match in order to go
                             first in the new suite
     :param exclude_pattern: pattern that names must not match, if any
     :param random_order:    if True, tests in the new suite will be put in
-                            random order
+                            random order (with all tests matching pattern
+                            first).
     :param append_rest:     if False, pattern is a strict filter and not
                             just an ordering directive
     :returns: the newly created suite
     """ 
-    first = []
-    second = []
+    if exclude_pattern is not None:
+        suite = exclude_tests_by_re(suite, exclude_pattern)
+    if random_order:
+        order_changer = randomize_suite
+    else:
+        order_changer = preserve_input
+    if append_rest:
+        suites = map(order_changer, split_suite_by_re(suite, pattern))
+        return TestUtil.TestSuite(suites)
+    else:
+        return order_changer(filter_suite_by_re(suite, pattern))
+
+
+def split_suite_by_re(suite, pattern):
+    """Split a test suite into two by a regular expression.
+    
+    :param suite: The suite to split.
+    :param pattern: A regular expression string. Test ids that match this
+        pattern will be in the first test suite returned, and the others in the
+        second test suite returned.
+    :return: A tuple of two test suites, where the first contains tests from
+        suite matching pattern, and the second contains the remainder from
+        suite. The order within each output suite is the same as it was in
+        suite.
+    """ 
+    matched = []
+    did_not_match = []
     filter_re = re.compile(pattern)
-    if exclude_pattern is not None:
-        exclude_re = re.compile(exclude_pattern)
     for test in iter_suite_tests(suite):
         test_id = test.id()
-        if exclude_pattern is None or not exclude_re.search(test_id):
-            if filter_re.search(test_id):
-                first.append(test)
-            elif append_rest:
-                second.append(test)
-    if random_order:
-        random.shuffle(first)
-        random.shuffle(second)
-    return TestUtil.TestSuite(first + second)
+        if filter_re.search(test_id):
+            matched.append(test)
+        else:
+            did_not_match.append(test)
+    return TestUtil.TestSuite(matched), TestUtil.TestSuite(did_not_match)
 
 
 def run_suite(suite, name='test', verbose=False, pattern=".*",
@@ -2320,14 +2457,20 @@
             (random_seed))
         random.seed(random_seed)
     # Customise the list of tests if requested
-    if pattern != '.*' or exclude_pattern is not None or random_order:
+    if exclude_pattern is not None:
+        suite = exclude_tests_by_re(suite, exclude_pattern)
+    if random_order:
+        order_changer = randomize_suite
+    else:
+        order_changer = preserve_input
+    if pattern != '.*' or random_order:
         if matching_tests_first:
-            suite = sort_suite_by_re(suite, pattern, exclude_pattern,
-                random_order)
+            suites = map(order_changer, split_suite_by_re(suite, pattern))
+            suite = TestUtil.TestSuite(suites)
         else:
-            suite = filter_suite_by_re(suite, pattern, exclude_pattern,
-                random_order)
+            suite = order_changer(filter_suite_by_re(suite, pattern))
 
+    # Activate code coverage.
     if coverage_dir is not None:
         tracer = trace.Trace(count=1, trace=0)
         sys.settrace(tracer.globaltrace)
@@ -2534,10 +2677,6 @@
     from bzrlib.tests.test_transport_implementations import TransportTestProviderAdapter
     adapter = TransportTestProviderAdapter()
     adapt_modules(test_transport_implementations, adapter, loader, suite)
-    adapt_tests(
-        ["bzrlib.tests.test_msgeditor.MsgEditorTest."
-         "test__create_temp_file_with_commit_template_in_unicode_dir"],
-        EncodingTestAdapter(), loader, suite)
     for package in packages_to_test():
         suite.addTest(package.test_suite())
     for m in MODULES_TO_TEST:

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2007-11-28 00:35:19 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2007-12-19 08:12:34 +0000
@@ -880,10 +880,10 @@
     module_name_list = [prefix + module_name
                         for module_name in test_repository_modules]
 
-    # Parameterise repository_implementations test modules by format.
+    # Parameterize repository_implementations test modules by format.
     result = multiply_tests_from_modules(module_name_list, format_scenarios)
 
-    # test_check_reconcile needs to be parameterised by format *and* by broken
+    # test_check_reconcile needs to be parameterized by format *and* by broken
     # repository scenario.
     broken_scenarios = [(s.__name__, {'scenario_class': s})
                         for s in all_broken_scenario_classes]

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2007-12-18 20:57:34 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-12-19 08:12:34 +0000
@@ -371,7 +371,7 @@
     def test_format_attributes(self):
         """All repository formats should have some basic attributes."""
         # create a repository to get a real format instance, not the 
-        # template from the test suite parameterisation.
+        # template from the test suite parameterization.
         repo = self.make_repository('.')
         repo._format.rich_root_data
         repo._format.supports_tree_reference

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2007-12-05 03:26:37 +0000
+++ bzrlib/tests/test_msgeditor.py	2007-12-19 08:12:34 +0000
@@ -33,14 +33,26 @@
     edit_commit_message_encoded
 )
 from bzrlib.tests import (
+    iter_suite_tests,
     probe_bad_non_ascii,
+    split_suite_by_re,
     TestCaseWithTransport,
     TestNotApplicable,
     TestSkipped,
     )
+from bzrlib.tests.EncodingAdapter import EncodingTestAdapter
 from bzrlib.trace import mutter
 
 
+def load_tests(standard_tests, module, loader):
+    """Parameterize the test for tempfile creation with different encodings."""
+    to_adapt, result = split_suite_by_re(standard_tests,
+        "test__create_temp_file_with_commit_template_in_unicode_dir")
+    for test in iter_suite_tests(to_adapt):
+        result.addTests(EncodingTestAdapter().adapt(test))
+    return result
+
+
 class MsgEditorTest(TestCaseWithTransport):
 
     def make_uncommitted_tree(self):

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2007-10-26 08:12:56 +0000
+++ bzrlib/tests/test_selftest.py	2007-12-19 08:12:34 +0000
@@ -36,9 +36,10 @@
 from bzrlib.progress import _BaseProgressBar
 from bzrlib.repofmt import weaverepo
 from bzrlib.symbol_versioning import (
-        zero_ten,
-        zero_eleven,
-        )
+    one_zero,
+    zero_eleven,
+    zero_ten,
+    )
 from bzrlib.tests import (
                           ChrootedTestCase,
                           ExtendedTestResult,
@@ -54,9 +55,17 @@
                           TestUtil,
                           TextTestRunner,
                           UnavailableFeature,
+                          condition_id_re,
+                          condition_isinstance,
+                          exclude_tests_by_condition,
+                          exclude_tests_by_re,
+                          filter_suite_by_condition,
+                          filter_suite_by_re,
                           iter_suite_tests,
-                          filter_suite_by_re,
+                          preserve_input,
+                          randomize_suite,
                           sort_suite_by_re,
+                          split_suite_by_re,
                           test_lsprof,
                           test_suite,
                           )
@@ -521,7 +530,7 @@
         # because each optimiser can be direction specific, we need to test
         # each optimiser in its chosen direction.
         # unlike the TestProviderAdapter we dont want to automatically add a
-        # parameterised one for WorkingTree - the optimisers will tell us what
+        # parameterized one for WorkingTree - the optimisers will tell us what
         # ones to add.
         from bzrlib.tests.tree_implementations import (
             return_parameter,
@@ -1660,21 +1669,104 @@
         self.loader = TestUtil.TestLoader()
         self.suite.addTest(self.loader.loadTestsFromModuleNames([
             'bzrlib.tests.test_selftest']))
-        self.all_names = [t.id() for t in iter_suite_tests(self.suite)]
+        self.all_names = self._test_ids(self.suite)
+
+    def _test_ids(self, test_suite):
+        """Get the ids for the tests in a test suite."""
+        return [t.id() for t in iter_suite_tests(test_suite)]
+
+    def test_condition_id_re(self):
+        test_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_condition_id_re')
+        filtered_suite = filter_suite_by_condition(self.suite,
+            condition_id_re('test_condition_id_re'))
+        self.assertEqual([test_name], self._test_ids(filtered_suite))
+
+    def test_condition_isinstance(self):
+        filtered_suite = filter_suite_by_condition(self.suite,
+            condition_isinstance(self.__class__))
+        class_pattern = 'bzrlib.tests.test_selftest.TestSelftestFiltering.'
+        re_filtered = filter_suite_by_re(self.suite, class_pattern)
+        self.assertEqual(self._test_ids(re_filtered),
+            self._test_ids(filtered_suite))
+
+    def test_exclude_tests_by_condition(self):
+        excluded_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_exclude_tests_by_condition')
+        filtered_suite = exclude_tests_by_condition(self.suite,
+            lambda x:x.id() == excluded_name)
+        self.assertEqual(len(self.all_names) - 1,
+            filtered_suite.countTestCases())
+        self.assertFalse(excluded_name in self._test_ids(filtered_suite))
+        remaining_names = list(self.all_names)
+        remaining_names.remove(excluded_name)
+        self.assertEqual(remaining_names, self._test_ids(filtered_suite))
+
+    def test_exclude_tests_by_re(self):
+        self.all_names = self._test_ids(self.suite)
+        filtered_suite = exclude_tests_by_re(self.suite, 'exclude_tests_by_re')
+        excluded_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_exclude_tests_by_re')
+        self.assertEqual(len(self.all_names) - 1,
+            filtered_suite.countTestCases())
+        self.assertFalse(excluded_name in self._test_ids(filtered_suite))
+        remaining_names = list(self.all_names)
+        remaining_names.remove(excluded_name)
+        self.assertEqual(remaining_names, self._test_ids(filtered_suite))
+
+    def test_filter_suite_by_condition(self):
+        test_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_filter_suite_by_condition')
+        filtered_suite = filter_suite_by_condition(self.suite,
+            lambda x:x.id() == test_name)
+        self.assertEqual([test_name], self._test_ids(filtered_suite))
 
     def test_filter_suite_by_re(self):
-        filtered_suite = filter_suite_by_re(self.suite, 'test_filter')
-        filtered_names = [t.id() for t in iter_suite_tests(filtered_suite)]
+        filtered_suite = filter_suite_by_re(self.suite, 'test_filter_suite_by_r')
+        filtered_names = self._test_ids(filtered_suite)
         self.assertEqual(filtered_names, ['bzrlib.tests.test_selftest.'
             'TestSelftestFiltering.test_filter_suite_by_re'])
-            
+
+    def test_preserve_input(self):
+        # NB: Surely this is something in the stdlib to do this?
+        self.assertTrue(self.suite is preserve_input(self.suite))
+        self.assertTrue("@#$" is preserve_input("@#$"))
+
+    def test_randomize_suite(self):
+        randomized_suite = randomize_suite(self.suite)
+        # randomizing should not add or remove test names.
+        self.assertEqual(set(self._test_ids(self.suite)),
+            set(self._test_ids(randomized_suite)))
+        # Technically, this *can* fail, because random.shuffle(list) can be
+        # equal to list. Trying multiple times just pushes the frequency back.
+        # As its len(self.all_names)!:1, the failure frequency should be low
+        # enough to ignore. RBC 20071021.
+        # It should change the order.
+        self.assertNotEqual(self.all_names, self._test_ids(randomized_suite))
+        # But not the length. (Possibly redundant with the set test, but not
+        # necessarily.)
+        self.assertEqual(len(self.all_names),
+            len(self._test_ids(randomized_suite)))
+
     def test_sort_suite_by_re(self):
-        sorted_suite = sort_suite_by_re(self.suite, 'test_filter')
-        sorted_names = [t.id() for t in iter_suite_tests(sorted_suite)]
+        sorted_suite = self.applyDeprecated(one_zero,
+            sort_suite_by_re, self.suite, 'test_filter_suite_by_r')
+        sorted_names = self._test_ids(sorted_suite)
         self.assertEqual(sorted_names[0], 'bzrlib.tests.test_selftest.'
             'TestSelftestFiltering.test_filter_suite_by_re')
         self.assertEquals(sorted(self.all_names), sorted(sorted_names))
 
+    def test_split_suit_by_re(self):
+        self.all_names = self._test_ids(self.suite)
+        split_suite = split_suite_by_re(self.suite, 'test_filter_suite_by_r')
+        filtered_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_filter_suite_by_re')
+        self.assertEqual([filtered_name], self._test_ids(split_suite[0]))
+        self.assertFalse(filtered_name in self._test_ids(split_suite[1]))
+        remaining_names = list(self.all_names)
+        remaining_names.remove(filtered_name)
+        self.assertEqual(remaining_names, self._test_ids(split_suite[1]))
+
 
 class TestCheckInventoryShape(TestCaseWithTransport):
 
@@ -1714,3 +1806,38 @@
         out, err = self.run_bzr(["log", "/nonexistantpath"], retcode=3)
         self.assertEqual(out, '')
         self.assertEqual(err, 'bzr: ERROR: Not a branch: "/nonexistantpath/".\n')
+
+
+class TestTestLoader(TestCase):
+    """Tests for the test loader."""
+
+    def _get_loader_and_module(self):
+        """Gets a TestLoader and a module with one test in it."""
+        loader = TestUtil.TestLoader()
+        module = {}
+        class Stub(TestCase):
+            def test_foo(self):
+                pass
+        class MyModule(object):
+            pass
+        MyModule.a_class = Stub
+        module = MyModule()
+        return loader, module
+
+    def test_module_no_load_tests_attribute_loads_classes(self):
+        loader, module = self._get_loader_and_module()
+        self.assertEqual(1, loader.loadTestsFromModule(module).countTestCases())
+
+    def test_module_load_tests_attribute_gets_called(self):
+        loader, module = self._get_loader_and_module()
+        # 'self' is here because we're faking the module with a class. Regular
+        # load_tests do not need that :)
+        def load_tests(self, standard_tests, module, loader):
+            result = loader.suiteClass()
+            for test in iter_suite_tests(standard_tests):
+                result.addTests([test, test])
+            return result
+        # add a load_tests() method which multiplies the tests from the module.
+        module.__class__.load_tests = load_tests
+        self.assertEqual(2, loader.loadTestsFromModule(module).countTestCases())
+

